;Phone Card Reader
;=================

;Display the remaining amount of money on a 2nd generation phone card
;ATTiny11 @ 1 MHz (internal RC-Oscillator)

;Ports:
;  PB0 = Shift Register Data (bits 1..7 = display g..a, bit 0 = switch 1's/10's display)
;  PB1 = Shift Register Clock             ^[common cathode]            ^0  ^1
;  PB2 = Select Euro Display
;  PB3 = Phone Card Reset
;  PB4 = Select Cent Display / Phone Card Clock
;  PB5 = Phone Card I/O (external pull-up 4.7k) ==> set RSTDISBL fuse to 0 (programmed)!

; Phone Card Connector Pinout:
;  _______________
; / +   \   , GND \
; |______\ /______|
; | RST |   | NC  |
; |_____|_ _|_____|
; | CLK  / \  I/O |
; \_____/___\_____/

;Versions:
;  0.1  2006-06-16/18  Arne Rossius

.include "../include/tn11def.inc"

.def	temp = R16
.def	temp2 = R17
.def	count = R18
.def	anz1 = R19
.def	anz2 = R20
.def	anz3 = R21
.def	anz4 = R22

.equ	DispDat = 0
.equ	DispClk = 1
.equ	DispEuro = 2
.equ	DispCent = 4
.equ	CardClk = 4
.equ	CardDat = 5
.equ	CardRst = 3

;===============================================================================

.org 0x000
	rjmp	start

start:
	;PORTB setup
	ldi	temp, 0x1F
	out	DDRB, temp
	ldi	temp, 0x00
	out	PORTB, temp

	;testing: display "LOAd"
	ldi	anz1, 0x38 | 0x80
	ldi	anz2, 0x3F
	ldi	anz3, 0x77 | 0x80
	ldi	anz4, 0x5E



	;display colon
	ldi	temp, 0x80
	rcall	shift
	sbi	PORTB, DispEuro


	
	;read from phone card
	clr	ZL
	clr	ZH
	
	sbi	PORTB, CardRst ;Reset card
	ldi	count, 16
reset_loop:
	sbi	PORTB, CardClk
	cbi	PORTB, CardClk
	dec	count
	brne	reset_loop
	cbi	PORTB, CardRst

	rcall	readbyte ;8 unimportant bytes
	rcall	readbyte
	rcall	readbyte
	rcall	readbyte
	rcall	readbyte
	rcall	readbyte
	rcall	readbyte
	rcall	readbyte

	rcall	readbyte ;4096
	ldi	count, 12
	rcall	addamount
	rcall	readbyte ;512
	ldi	count, 9
	rcall	addamount
	rcall	readbyte ;64
	ldi	count, 6
	rcall	addamount
	rcall	readbyte ;8
	ldi	count, 3
	rcall	addamount
	rcall	readbyte
	ldi	count, 0
	rcall	addamount



	;divide amount into 4 decimal digits
	ldi	temp, LOW(1000)
	ldi	temp2, HIGH(1000)
	clr	anz1
div1000:
	cp	ZL, temp
	cpc	ZH, temp2
	brlo	div1000end
	sub	ZL, temp
	sbc	ZH, temp2
	inc	anz1
	rjmp	div1000
div1000end:

	ldi	temp, LOW(100)
	ldi	temp2, HIGH(100)
	clr	anz2
div100:
	cp	ZL, temp
	cpc	ZH, temp2
	brlo	div100end
	sub	ZL, temp
	sbc	ZH, temp2
	inc	anz2
	rjmp	div100
div100end:

	ldi	temp, 10
	clr	anz3
div10:
	cp	ZL, temp
	brlo	div10end
	sub	ZL, temp
	inc	anz3
	rjmp	div10
div10end:

	mov	anz4, ZL



	;encode digits to 7-segment
	mov	temp, anz1
	rcall	encode
	ori	temp, 0x80
	mov	anz1, temp

	mov	temp, anz2
	rcall	encode
	andi	temp, LOW(~0x80)
	mov	anz2, temp

	mov	temp, anz3
	rcall	encode
	ori	temp, 0x80
	mov	anz3, temp

	mov	temp, anz4
	rcall	encode
	andi	temp, LOW(~0x80)
	mov	anz4, temp



mux:
	;multiplex LED display
	mov	temp, anz1
	rcall	shift
	sbi	PORTB, DispEuro
	rcall	muxwait
	
	mov	temp, anz2
	rcall	shift
	sbi	PORTB, DispEuro
	rcall	muxwait
	
	mov	temp, anz3
	rcall	shift
	sbi	PORTB, DispCent
	rcall	muxwait

	mov	temp, anz4
	rcall	shift
	sbi	PORTB, DispCent
	rcall	muxwait

	rjmp	mux

;===============================================================================

readbyte: ;read 1 byte from the phone card and return number of set bits in it
	clr	temp
	ldi	count, 8
readbyte_loop:
	sbic	PINB, CardDat
	inc	temp
	sbi	PORTB, CardClk
	cbi	PORTB, CardClk
	dec	count
	brne	readbyte_loop
	ret

;-------------------

addamount: ;shift amount left (multiply by 2^count)
	clr	temp2
addamount_loop:
	cpi	count, 0
	breq	addamount_end
	lsl	temp
	rol	temp2
	dec	count
	rjmp	addamount_loop
addamount_end:

	add	ZL, temp
	adc	ZH, temp2
	ret

;-------------------

encode: ;encode number to 7-segment data
	ldi	ZL, LOW(encode_table*2)
	ldi	ZH, HIGH(encode_table*2)
	add	ZL, temp
	brcc	PC+2
	inc	ZH
	lpm
	mov	temp, R0
	ret

encode_table:
	.db 0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x6F

;-------------------


muxwait: ;slow down multiplexing
	ldi	temp, 0
muxwait_loop:
	dec	temp
	brne	muxwait_loop
	ret

;-------------------

shift: ;shift out one byte, LSB first
	ldi	count, 8
shift_loop:
	ldi	temp2, 0x00
	out	PORTB, temp2
	lsr	temp
	brcc	PC+2
	sbi	PORTB, DispDat
	sbi	PORTB, DispClk
	dec	count
	brne	shift_loop
	ret
